@@ -9,6 +9,7 @@ use ide_db::{
     SnippetCap,
     assists::ExprFillDefaultMode,
     imports::{import_assets::ImportPathConfig, insert_use::InsertUseConfig},
+    rename::RenameConfig,
 };
 
 use crate::AssistKind;
@@ -27,6 +28,7 @@ pub struct AssistConfig {
     pub code_action_grouping: bool,
     pub expr_fill_default: ExprFillDefaultMode,
     pub prefer_self_ty: bool,
+    pub show_rename_conflicts: bool,
 }
 
 impl AssistConfig {
@@ -38,7 +40,7 @@ impl AssistConfig {
         }
     }
 
-    pub fn find_path_confg(&self, allow_unstable: bool) -> FindPathConfig {
+    pub fn find_path_config(&self, allow_unstable: bool) -> FindPathConfig {
         FindPathConfig {
             prefer_no_std: self.prefer_no_std,
             prefer_prelude: self.prefer_prelude,
@@ -46,4 +48,8 @@ impl AssistConfig {
             allow_unstable,
         }
     }
+
+    pub fn rename_config(&self) -> RenameConfig {
+        RenameConfig { show_conflicts: self.show_rename_conflicts }
+    }
 }
