@@ -28,7 +28,6 @@ use ide_db::{
 use itertools::Itertools;
 use macros::UpmapFromRaFixture;
 use nohash_hasher::IntMap;
-use span::Edition;
 use syntax::AstToken;
 use syntax::{
     AstNode,
@@ -39,7 +38,8 @@ use syntax::{
 };
 
 use crate::{
-    Analysis, FilePosition, HighlightedRange, NavigationTarget, TryToNav, highlight_related,
+    Analysis, FilePosition, HighlightedRange, NavigationTarget, TryToNav,
+    doc_links::token_as_doc_comment, highlight_related,
 };
 
 /// Result of a reference search operation.
@@ -212,6 +212,13 @@ pub(crate) fn find_defs(
     syntax: &SyntaxNode,
     offset: TextSize,
 ) -> Option<Vec<Definition>> {
+    if let Some(token) = syntax.token_at_offset(offset).left_biased()
+        && let Some(doc_comment) = token_as_doc_comment(&token)
+    {
+        return doc_comment
+            .get_definition_with_descend_at(sema, offset, |def, _, _| Some(vec![def]));
+    }
+
     let token = syntax.token_at_offset(offset).find(|t| {
         matches!(
             t.kind(),
@@ -419,10 +426,7 @@ fn handle_control_flow_keywords(
     FilePosition { file_id, offset }: FilePosition,
 ) -> Option<ReferenceSearchResult> {
     let file = sema.parse_guess_edition(file_id);
-    let edition = sema
-        .attach_first_edition(file_id)
-        .map(|it| it.edition(sema.db))
-        .unwrap_or(Edition::CURRENT);
+    let edition = sema.attach_first_edition(file_id).edition(sema.db);
     let token = pick_best_token(file.syntax().token_at_offset(offset), |kind| match kind {
         _ if kind.is_keyword(edition) => 4,
         T![=>] => 3,
@@ -789,6 +793,23 @@ fn main() {
         );
     }
 
+    #[test]
+    fn test_find_all_refs_in_comments() {
+        check(
+            r#"
+struct Foo;
+
+/// $0[`Foo`] is just above
+struct Bar;
+"#,
+            expect![[r#"
+                Foo Struct FileId(0) 0..11 7..10
+
+                (no references)
+            "#]],
+        );
+    }
+
     #[test]
     fn search_filters_by_range() {
         check(
@@ -1058,7 +1079,7 @@ use self$0;
 use self$0;
 "#,
             expect![[r#"
-                _ Module FileId(0) 0..10
+                _ CrateRoot FileId(0) 0..10
 
                 FileId(0) 4..8 import
             "#]],
@@ -1124,7 +1145,10 @@ pub(super) struct Foo$0 {
         check_with_scope(
             code,
             Some(&mut |db| {
-                SearchScope::single_file(EditionedFileId::current_edition(db, FileId::from_raw(2)))
+                SearchScope::single_file(EditionedFileId::current_edition_guess_origin(
+                    db,
+                    FileId::from_raw(2),
+                ))
             }),
             expect![[r#"
                 quux Function FileId(0) 19..35 26..30
@@ -2504,7 +2528,7 @@ fn r#fn$0() {}
 fn main() { r#fn(); }
 "#,
             expect![[r#"
-                r#fn Function FileId(0) 0..12 3..7
+                fn Function FileId(0) 0..12 3..7
 
                 FileId(0) 25..29
             "#]],
